function BlocklistAssistant(){
    this.listModel = {
        items: []
    };
    _blocklistAssistant = this;
}

BlocklistAssistant.prototype.setup = function(){
    this.menuModel = {
        visible: true,
        items: []
    };
    if (_chatsAssistant != null && _appAssistant.isTouchPad()) {
        this.menuModel.items = [{
            icon: "back",
            command: "goBack"
        }, {
            icon: "search",
            command: "search"
        }];
    }
    
    this.controller.setupWidget(Mojo.Menu.commandMenu, this.attributes = {
        spacerHeight: 0,
        menuClass: 'no-fade'
    }, this.menuModel);
    
    this.controller.setupWidget(Mojo.Menu.appMenu, this.attributesAppMenu = {
        omitDefaultItems: true
    }, this.appMenuModel = {
        visible: true,
        items: [{
            label: $L("Emtpy list"),
            command: "empty"
        }, {
            label: $L("Settings"),
            items: [{
                label: $L("Preferences"),
                command: Mojo.Menu.prefsCmd
            }, {
                label: $L("Account settings"),
                command: "accountSettings"
            }, {
                label: $L("Help"),
                command: Mojo.Menu.helpCmd
            }]
        }, {
            label: $L("Exit"),
            command: "exit"
        }]
    });
    
    this.controller.get("headerTitle").update($L("Blocked contacts"));
    
    this.listElement = this.controller.get("contactList");
    this.controller.setupWidget("contactList", {
        swipeToDelete: true,
        hasNoWidgets: true,
        addItemLabel: $L("Add contact"),
        itemTemplate: "blocklist/list-entry",
        onItemRendered: function(listWidget, itemModel, itemNode){
            try {
                if (itemModel.iswa && !itemModel.rendered) {
                    // _plugin.sendGetPictureIds(JSON.stringify([itemModel.jid]));
                    _plugin.sendGetPicture(itemModel.jid, "preview", (itemModel.pictureid ? itemModel.pictureid : ""), "");
                    itemModel.rendered = true;
                }
            } 
            catch (e) {
                Mojo.Log.error("error onItemRendered: %j", e);
            }
        }
.bind(this)        ,
        formatters: {
            icon: function(value, model){
                if (model.picturepath) {
                    var imgSize = '60:60';
                    if (_appAssistant.isPre3()) 
                        imgSize = '90:90';
                    return '<div class="contact-list-icon picture icon left" style="background-image:url(/var/luna/data/extractfs' + encodeURIComponent(model.picturepath) + ':0:0:' + imgSize + ':3)"></div>';
                }
                else {
                    return '<div class="contact-list-icon person-icon icon left"></div>';
                }
            }
        }
    }, this.listModel);
    
    
    this.controller.setupWidget("sendChangesButton", {
        label: $L("Send changes"),
        type: Mojo.Widget.activityButton
    }, this.buttonModel = {
        buttonClass: "affirmative",
        disabled: false
    });
    
    this.controller.listen('contactList', Mojo.Event.listAdd, this.addListHandler.bindAsEventListener(this));
    this.controller.listen('contactList', Mojo.Event.listDelete, this.deleteListHandler.bindAsEventListener(this));
    this.controller.listen('sendChangesButton', Mojo.Event.tap, this.sendChangesHandler.bindAsEventListener(this));
};


BlocklistAssistant.prototype.ready = function(event){
    _appDB.getBlockedContacts(function(contacts){
        this.listModel.items = contacts;
        this.controller.modelChanged(this.listModel);
    }
.bind(this));
}

BlocklistAssistant.prototype.deleteListHandler = function(event){
    event.item.deleted = true;
}

BlocklistAssistant.prototype.addListHandler = function(event){
    _appDB.getAllContacts(function(contacts){
        this.controller.stageController.pushScene('contact-list', contacts, true);
    }
.bind(this));
}

BlocklistAssistant.prototype.sendChangesHandler = function(event){
    this.buttonModel.disabled = true;
    this.controller.modelChanged(this.buttonModel);
    this.buttonSpinning = true;
    _mojowhatsupPlugin.safePluginCall(function(){
        var jids = [];
        for (var i = 0; i < this.listModel.items.length; i++) {
            if (!this.listModel.items[i].deleted && this.listModel.items[i].jid != _myJid) {
                jids.push(this.listModel.items[i].jid);
            }
        }
        _plugin.sendSetPrivacyList(JSON.stringify(jids));
        this.timeout = setTimeout(function(){
            this.onSetPrivacyListResult("error");
        }
.bind(this), 10000);
    }
.bind(this));
}

BlocklistAssistant.prototype.onSetPrivacyListResult = function(value){
    if (this.timeout) {
        clearTimeout(this.timeout);
    }
    this.controller.get("sendChangesButton").mojo.deactivate();
    this.buttonModel.disabled = false;
    this.controller.modelChanged(this.buttonModel);
    this.buttonSpinning = false;
    this.timeout = undefined;
    
    var msg = "";
    if (value == "ok") {
        msg = $L("Changes successfully completed");
        _appDB.clearBlockedContacts(function(){
            for (var i = 0; i < this.listModel.items.length; i++) {
                if (!this.listModel.items[i].deleted && this.listModel.items[i].jid != _myJid) {
                    _appDB.updateContactBlocked(this.listModel.items[i].jid, true);
                }
            }
        }
.bind(this));
    }
    else {
        msg = $L("There was an error. Try again later.")
    }
    
    this.controller.showAlertDialog({
        title: $L("Notification"),
        onChoose: function(){
            this.controller.stageController.popScene();
        }
.bind(this)        ,
        message: msg,
        choices: [{
            label: $L("OK")
        }]
    });
}

BlocklistAssistant.prototype.handleCommand = function(event){
    if (event.type == Mojo.Event.back && this.buttonSpinning) {
        event.stop();
        return;
    }
    if (event.type == Mojo.Event.command) {
        switch (event.command) {
            case 'empty':
                this.listModel.items = [];
                this.controller.modelChanged(this.listModel);
                break;
            case 'goBack':
                this.controller.stageController.popScene();
                break;
            case Mojo.Menu.prefsCmd:
                this.controller.stageController.pushScene("prefs");
                break;
            case "accountSettings":
                this.controller.stageController.pushScene("account");
                break;
            case Mojo.Menu.helpCmd:
                this.controller.stageController.pushScene("help");
                break;
            case "exit":
                _exitApp = true;
                Mojo.Controller.getAppController().closeStage(_mainStage);
                break;
        }
    }
}

BlocklistAssistant.prototype.updatetPicture = function(jid, picturepath){
    for (var i = 0; i < this.listModel.items.length; i++) {
        if (this.listModel.items[i].jid == jid) {
            this.listModel.items[i].picturepath = picturepath;
            this.listElement.mojo.noticeUpdatedItems(i, [this.listModel.items[i]]);
            return;
        }
    }
}

BlocklistAssistant.prototype.activate = function(param){
    if (param && ("jid" in param)) {
        for (var i = 0; i < this.listModel.items.length; i++) 
            if (this.listModel.items[i].jid == param.jid && !this.listModel.items[i].deleted) 
                return;
        this.listModel.items.push(param);
        this.listElement.mojo.noticeAddedItems(this.listModel.items.length - 1, [param]);
    }
};

BlocklistAssistant.prototype.deactivate = function(event){
	if (_contactsAssistant != null) {
		_contactsAssistant.loadContacts();
	}
	
	if (_broadcastAssistant != null) {
		_broadcastAssistant.loadChats();
	}
};

BlocklistAssistant.prototype.cleanup = function(event){
    _blocklistAssistant = null;
    
    this.controller.stopListening('contactList', Mojo.Event.listAdd, this.addListHandler);
    this.controller.stopListening('contactList', Mojo.Event.listDelete, this.deleteListHandler);
    this.controller.stopListening('sendChangesButton', Mojo.Event.tap, this.sendChangesHandler);
    
};

