function GroupAssistant(group, chat, mode){
    _groupAssistant = this;
    this.mode = mode;
    this.group = group;
    this.chat = chat;
    this.participantsListModel = {
        items: []
    };
    this.addedParticipants = [];
    this.removedParticipants = [];
}

GroupAssistant.prototype.setup = function(){
    if (_chatsAssistant != null && _appAssistant.isTouchPad()) {
        var menuModel = {
            visible: true,
            items: [{
                icon: "back",
                command: "goBack"
            }]
        };
        
        this.controller.setupWidget(Mojo.Menu.commandMenu, this.attributes = {
            spacerHeight: 0,
            menuClass: 'no-fade'
        }, menuModel);
    }
    var headerTitle = "";
    switch (this.mode) {
        case 'new':
            headerTitle = "New group";
            break;
        case 'edit':
            headerTitle = "Group properties";
            break;
        case 'info':
            headerTitle = "Group properties";
            break;
    }
    
    
    this.controller.get('headerTitle').update($L(headerTitle));
    this.controller.get('groupInfoTitle').update($L("Info"));
    this.controller.get('ownerLabel').update($L("Owner"));
    this.controller.get('creationLabel').update($L("Creation date"));
    
    this.controller.setupWidget("spinnerId", this.attributesSpinner = {
        spinnerSize: "small"
    }, this.modelSpinner = {
        spinning: false
    });
    
    this.controller.get('spinnerId').hide();
    this.setPicture();
    
    this.controller.listen("groupImage", Mojo.Event.tap, this.groupImageHandler.bindAsEventListener(this));
    
    if (this.group != null) {
        var owner = Message.removeDomainFromJid(this.group.owner);
        if (_contactJidNames.getItem(this.group.owner) !== undefined) {
            owner = _contactJidNames.getItem(this.group.owner);
        }
        this.controller.get('ownerValue').update(owner);
        // this.controller.get('ownerListValue').update(owner);
        this.controller.get('creationValue').update(Mojo.Format.formatDate(new Date(this.group.creation_t * 1000), 'short'));
    }
    
    this.controller.setupWidget('textFieldSubject', this.attributesTF = {
        hintText: $L("Insert a subject") + " ...",
        autofocus: true,
        multiline: false,
        modifierState: Mojo.Widget.shiftSingle,
        enterSubmits: true,
        focusMode: Mojo.Widget.focusInsertMode,
        maxLength: _appData.cookieData.max_subject,
        changeOnKeyPress: true
    }, this.modelTF = {
        value: (this.group ? this.group.subject : ""),
        // disabled: (this.mode == 'info'? true : false)
    });
    
    
    this.controller.listen('textFieldSubject', Mojo.Event.propertyChanged, this.subjectChangedHandler.bindAsEventListener(this));
    
    this.setSubjectTitle((this.group ? this.group.subject : ""));
    
    // if (this.mode == 'info') {
    // this.controller.get("emojiButton").hide();
    // }
    
    this.controller.listen('emojiButton', Mojo.Event.tap, this.emojiButtonHandler.bindAsEventListener(this));
    
    this.controller.setupWidget("sendChangesButton", {
        label: $L("Send changes")
    }, {
        value: "",
        buttonClass: "affirmative",
        disabled: false
    });
    this.controller.listen('sendChangesButton', Mojo.Event.tap, this.sendChangesHandler.bindAsEventListener(this));
    
    // if (this.mode == 'info') {
    // this.controller.get("sendChangesButton").hide();
    // }
    
    var removeButtonLabel = $L("End group");
    if (this.mode == 'info') {
        removeButtonLabel = $L("Leave group");
    }
    this.controller.setupWidget("removeGroupButton", {
        label: removeButtonLabel
    }, {
        value: "",
        buttonClass: "negative",
        disabled: false
    });
    this.controller.listen('removeGroupButton', Mojo.Event.tap, this.removeGroupHandler.bindAsEventListener(this));
    
    if (this.mode == 'new') {
        this.controller.get("removeGroupButton").hide();
    }
    
    this.participantsListAttributes = {
        itemTemplate: "group/participant-list-entry",
        swipeToDelete: (this.mode == 'edit' ? true : false),
        hasNoWidgets: true,
        fixedHeightItems: true,
        preventDeleteProperty: "cannotdelete",
        onItemRendered: function(listWidget, itemModel, itemNode){
            try {
                if (!itemModel.rendered) {
                    // _plugin.sendGetPictureIds(JSON.stringify([itemModel.jid]));
                    _plugin.sendGetPicture(itemModel.jid, "preview", (itemModel.pictureid ? itemModel.pictureid : ""), "");
                    itemModel.rendered = true;
                }
            } 
            catch (e) {
                Mojo.Log.error("error onItemRendered: %j", e);
            }
        },
        formatters: {
            icon: function(value, model){
                if (model.picturepath) {
                    var imgSize = '60:60';
                    if (_appAssistant.isPre3()) 
                        imgSize = '90:90';
                    return '<div class="contact-list-icon picture icon left" style="background-image:url(/var/luna/data/extractfs' + encodeURIComponent(model.picturepath) + ':0:0:' + imgSize + ':3)"></div>';
                }
                else {
                    return '<div class="contact-list-icon person-icon icon left"></div>';
                }
            },
            name: function(value, model){
                if (model.name) {
                    return model.name;
                }
                else 
                    if (model.jid) 
                        return "+" + Message.removeDomainFromJid(model.jid);
                return "";
            },
            status: function(value, model){
                if (model.status) {
                    return emojify(model.status, 18);
                }
                else 
                    return "";
            }
        }
    };
    
    if (this.mode == 'edit') {
        this.participantsListAttributes.addItemLabel = $L("Add contact");
    }
    
    this.controller.setupWidget("participantsList", this.participantsListAttributes, this.participantsListModel);
    
    if (this.mode == 'edit') {
        this.controller.listen('participantsList', Mojo.Event.listAdd, this.addListHandler.bindAsEventListener(this));
        this.controller.listen('participantsList', Mojo.Event.listDelete, this.deleteListHandler.bindAsEventListener(this));
    }
    
    this.controller.listen('participantsList', Mojo.Event.listTap, this.listTapHandler.bindAsEventListener(this));
    
    if (this.mode == 'new') {
        this.controller.get('listGroup').hide();
        this.controller.get('infoGroup').hide();
    }
    
    this.controller.get('preview').update(emojify((this.group ? this.group.subject : ""), 20));
    this.controller.listen('textFieldSubject', Mojo.Event.propertyChange, this.textChangedHandler.bindAsEventListener(this));
}


GroupAssistant.prototype.ready = function(event){
    if (this.group) {
        this.participants = this.group.getParticipantsJidsArray();
        _appDB.getTheseContacts(this.participants, function(contacts){
            for (var i = 0; i < contacts.length; i++) {
                if (contacts[i].jid == this.group.owner) {
                    contacts[i].cannotdelete = true;
                }
                this.participants[this.participants.indexOf(contacts[i].jid)] = null;
            }
            for (var i = 0; i < this.participants.length; i++) {
                if (this.participants[i] != null) {
                    var obj = {
                        jid: this.participants[i]
                    };
                    if (obj.jid == this.group.owner) {
                        obj.cannotdelete = true;
                    }
                    contacts.push(obj);
                }
            }
            
            this.participantsListModel.items = contacts;
            this.controller.get('participantsList').mojo.noticeUpdatedItems(0, this.participantsListModel.items);
            this.setParticipantsTitle();
        }
.bind(this));
    }
}

GroupAssistant.prototype.textChangedHandler = function(event){
    var text = this.controller.get('textFieldSubject').mojo.getValue();
    this.controller.get('preview').update(emojify(text, 20));
}

GroupAssistant.prototype.setPicture = function(){
    var imgSize = '80:80';
    if (_appAssistant.isPre3()) 
        imgSize = '120:120';
    
    if ((this.mode != "new") && this.chat.picturepath) {
        this.controller.get('groupImage').update('<div class="group-image picture" style="background-image:url(/var/luna/data/extractfs' + encodeURIComponent(this.chat.picturepath) + ':0:0:' + imgSize + ':3)"></div>');
    }
    else 
        if (this.mode != "new") {
            this.controller.get('groupImage').update('<div class="group-image group-icon"></div>');
        }
        else {
            this.controller.get('groupImage').update('<div style="display: none"></div>');
        }
}

GroupAssistant.prototype.updatePicture = function(jid, path){
    if (this.chat.jid == jid) {
        if (path != "error") {
            this.chat.picturepath = path;
            this.setPicture();
        }
        
        this.modelSpinner.spinning = false;
        this.controller.modelChanged(this.modelSpinner);
        this.controller.get("spinnerId").hide();
    }
    else {
        for (var i = 0; i < this.participantsListModel.items.length; i++) {
            if (this.participantsListModel.items[i].jid == jid) {
                this.participantsListModel.items[i].picturepath = path;
                this.controller.get('participantsList').mojo.noticeUpdatedItems(i, [this.participantsListModel.items[i]]);
                return;
            }
        }
    }
}

GroupAssistant.prototype.subjectChangedHandler = function(event){
    var text = this.controller.get('textFieldSubject').mojo.getValue();
    this.setSubjectTitle(text);
}

GroupAssistant.prototype.groupImageHandler = function(event){
    Event.stop(event);
    var items = [];
    items.push({
        label: $L("Change"),
        command: "change"
    });
    
    if (this.chat.picturepath) {
        items.push({
            label: $L("View"),
            command: "view"
        });
        items.push({
            label: $L("Delete"),
            command: "delete"
        });
    }
    
    this.controller.popupSubmenu({
        onChoose: this.popupImageHandler,
        placeNear: this.controller.get("groupImage"),
        items: items
    });
}

GroupAssistant.prototype.popupImageHandler = function(command){
    switch (command) {
        case 'change':
            var params = {
                kind: "image",
                actionName: $L("Send"),
                extensions: ["jpg"],
                crop: {
                    width: 320,
                    height: 320
                },
                onSelect: function(file){
                    this.modelSpinner.spinning = true;
                    this.controller.modelChanged(this.modelSpinner);
                    this.controller.get("spinnerId").show();
                    Mojo.Log.info("Crop info: %j", file.cropInfo);
                    _mojowhatsupPlugin.safePluginCall(function(){
                        var cropInfo = {
                            size: 480,
                            scale: file.cropInfo.window.scale * 1.5,
                            x: Math.floor(file.cropInfo.window.suggestedXTop * file.cropInfo.window.scale * 1.5),
                            y: Math.floor(file.cropInfo.window.suggestedYTop * file.cropInfo.window.scale * 1.5)
                        };
                        _plugin.sendSetPicture(this.chat.jid, file.fullPath, JSON.stringify(cropInfo));
                    }
.bind(this));
                }
.bind(this)
            };
            Mojo.FilePicker.pickFile(params, this.controller.stageController);
            break;
        case 'view':
            this.controller.stageController.pushScene("imageview", {
                path: this.chat.picturepath
            });
            break;
        case 'delete':
            this.modelSpinner.spinning = true;
            this.controller.modelChanged(this.modelSpinner);
            this.controller.get("spinnerId").show();
            
            _mojowhatsupPlugin.safePluginCall(function(){
                _plugin.sendSetPicture(this.chat.jid, "", "");
            }
.bind(this));
            break;
    }
}

GroupAssistant.prototype.setParticipantsTitle = function(){
    if (this.mode == 'edit') 
        this.controller.get('groupParticipantsTitle').update($L("Participants") + " (" + (this.participantsListModel.items.length) + " " + $L("of") + " " + _appData.cookieData.max_participants + ")");
    else {
        this.controller.get('groupParticipantsTitle').update($L("Participants") + " (" + (this.participantsListModel.items.length) + ")");
    }
}

GroupAssistant.prototype.setSubjectTitle = function(subject){
    var left = (_appData.cookieData.max_subject - subject.length);
    this.controller.get('subjectTitle').update($L("Subject") + " (" +
    $L("#{left} remaining").interpolate({
        left: left
    }) +
    ")");
}

GroupAssistant.prototype.emojiButtonHandler = function(event){
    var text = this.controller.get('textFieldSubject').mojo.getValue();
    if (text.length < _appData.cookieData.max_subject) 
        this.controller.stageController.pushScene("emoji-dialog");
    else {
        Event.stop(event);
        this.controller.get('textFieldSubject').mojo.focus();
    }
}

GroupAssistant.prototype.listTapHandler = function(event){
    var contact = event.item;
    if (contact.jid != _myJid) {
        var name = contact.name;
        if (!contact.name) {
            name = "+" + Message.removeDomainFromJid(contact.jid);
        }
        
        var options = [{
            label: $L("Chat with") + " " + name,
            command: "newChat"
        }];
        
        if (!contact.name) {
            options.push({
                label: $L("Add to contacts"),
                command: "add"
            });
        }
        
        
        this.controller.popupSubmenu({
            onChoose: function(command){
                switch (command) {
                    case 'newChat':
                        _appDB.findOrCreateChat(contact.jid, function(chat, created){
                            this.controller.stageController.popScenesTo('chats-list');
                            this.controller.stageController.pushScene("chat", chat);
                            if (created) 
                                setTimeout(_appAssistant.notifyUpdateChats.bind(_appAssistant), 500);
                        }
.bind(this));
                        break;
                    case 'add':
                        var c = {
                            name: {
                                givenName: name
                            },
                            phoneNumbers: [{
                                value: name,
                                type: "type_mobile",
                                primary: true
                            }],
                        };
                        
                        this.controller.serviceRequest("palm://com.palm.applicationManager", {
                            method: "open",
                            parameters: {
                                id: "com.palm.app.contacts",
                                params: {
                                    contact: c,
                                    launchType: "newContact"
                                }
                            }
                        });
                        break;
                }
            }
.bind(this)            ,
            placeNear: event.originalEvent.target,
            items: options
        });
    }
}

GroupAssistant.prototype.deleteListHandler = function(event){
    this.participantsListModel.items.splice(event.index, 1);
    this.setParticipantsTitle();
    
    var p = event.item;
    if (this.group.existsParticipant(p.jid)) {
        this.removedParticipants.push(p.jid);
    }
    
    var i = this.addedParticipants.indexOf(p.jid);
    if (i != -1) {
        this.addedParticipants.splice(i, 1);
    }
}

GroupAssistant.prototype.addListHandler = function(event){
    if ((this.participantsListModel.items.length) < _appData.cookieData.max_participants) {
        _appDB.getAllContacts(function(contacts){
            this.controller.stageController.pushScene('contact-list', contacts, true);
        }
.bind(this));
    }
}

GroupAssistant.prototype.activate = function(param){
    if (param && ("jid" in param)) {
        if (!this.group.existsParticipant(param.jid) && (this.addedParticipants.indexOf(param.jid) == -1)) {
            this.addedParticipants.push(param.jid);
            this.participantsListModel.items.push(param);
            this.controller.get('participantsList').mojo.noticeAddedItems(this.participantsListModel.items.length - 1, [param]);
            this.setParticipantsTitle();
        }
        else 
            if (this.removedParticipants.indexOf(param.jid) >= 0) {
                this.removedParticipants.splice(this.removedParticipants.indexOf(param.jid), 1);
                this.participantsListModel.items.push(param);
                this.controller.get('participantsList').mojo.noticeAddedItems(this.participantsListModel.items.length - 1, [param]);
                this.setParticipantsTitle();
            }
    }
    else 
        if (param && "selectedEmoji" in param) {
            if (param.selectedEmoji != null) {
                var text = this.controller.get('textFieldSubject').mojo.getValue();
                this.controller.get('textFieldSubject').mojo.setValue(text + convertUnicodeCodePointsToString(['0x' + param.selectedEmoji]));
            }
            this.controller.get('textFieldSubject').mojo.focus();
        }
}

GroupAssistant.prototype.removeGroupHandler = function(event){
    var message = $L("Are you sure you want to remove/end this group?");
    if (this.mode == 'info') 
        message = $L("Are you sure you want to remove/leave this group?");
    this.controller.showAlertDialog({
        onChoose: function(value){
            if (value == "yes") {
                try {
                    if (this.mode == 'edit') {
                        _plugin.sendEndGroup(this.group.gjid);
                    }
                    else 
                        if (this.mode == 'info') {
                            _plugin.sendLeaveGroup(this.group.gjid);
                        }
                    this.controller.stageController.popScenesTo("chats-list");
                    _appDB.deleteGroupAndChat(this.group, this.chat, function(){
                        _appAssistant.notifyUpdateChats(this);
                    }
.bind(this));
                } 
                catch (e) {
                    Mojo.Log.error("Error sending end/leave group! " + e);
                }
            }
        }
.bind(this)        ,
        title: $L("Confirm"),
        message: message,
        choices: [{
            label: $L("Yes"),
            value: "yes",
            type: "affirmative"
        }, {
            label: $L("No"),
            value: "no",
            type: "negative"
        }, ]
    });
}

GroupAssistant.prototype.sendChangesHandler = function(event){
    var subject = this.controller.get('textFieldSubject').mojo.getValue();
    if (subject != null && subject != "") {
        if (this.mode == 'new') {
            try {
                _plugin.sendCreateGroupChat(JSONstringifyEscapeUnicode(subject));
                this.controller.stageController.popScene();
            } 
            catch (e) {
                Mojo.Log.error("Error sending create group! " + e);
            }
        }
        else {
            var subjectChanged = (subject != this.group.subject);
            try {
                if (subjectChanged) {
                    _plugin.sendSetNewSubject(this.group.gjid, JSONstringifyEscapeUnicode(subject));
                }
                if (this.addedParticipants.length > 0) {
                    _plugin.sendAddParticipants(this.group.gjid, JSON.stringify(this.addedParticipants));
                }
                if (this.removedParticipants.length > 0) {
                    _plugin.sendRemoveParticipants(this.group.gjid, JSON.stringify(this.removedParticipants));
                }
                this.controller.stageController.popScene();
            } 
            catch (e) {
                Mojo.Log.error("Error sending changes to group!" + e);
            }
        }
    }
    else {
        this.controller.get('errorMessageText').update($L("Empty subject. Please insert a subject for the group"));
        this.controller.get('error_message').show();
    }
}

GroupAssistant.prototype.handleCommand = function(event){
    if (event.type == Mojo.Event.command) {
        switch (event.command) {
            case 'goBack':
                this.controller.stageController.popScene();
                break;
        }
    }
}

GroupAssistant.prototype.cleanup = function(){
    _groupAssistant = null;
    this.controller.stopListening('sendChangesButton', Mojo.Event.tap, this.sendChangesHandler);
    this.controller.stopListening('removeGroupButton', Mojo.Event.tap, this.removeGroupHandler);
    this.controller.stopListening('emojiButton', Mojo.Event.tap, this.emojiButtonHandler);
    this.controller.stopListening('textFieldSubject', Mojo.Event.propertyChanged, this.subjectChangedHandler);
    if (this.mode == 'edit') {
        this.controller.stopListening('participantsList', Mojo.Event.listAdd, this.addListHandler);
        this.controller.stopListening('participantsList', Mojo.Event.listDelete, this.deleteListHandler);
    }
    this.controller.stopListening("groupImage", Mojo.Event.tap, this.groupImageHandler);
    this.controller.stopListening('textFieldSubject', Mojo.Event.propertyChange, this.textChangedHandler);
    this.controller.stopListening('participantsList', Mojo.Event.listTap, this.listTapHandler);
}
