function Depot(){
    this.db = null;
    this.prefs = null;
}

Depot.prototype.load = function(onSuccess, reset){
    if (this.db == null) {
        this.db = new Mojo.Depot({
            name: "ext:settings",
            version: 1,
            replace: false
        }, function(){
            this.getPrefs(function(result){
                if (result != null && !reset) {
                    this.prefs = result;
                    this.setDefaultNewValues(onSuccess);
                }
                else 
                    if (_appData.isRegistered()) {
                        this.prefs = new PreferencesCookie().get();
                        this.setDefaultNewValues(onSuccess);
                    }
                    else {
                        this.putPrefs(this.getDefaultValues(), undefined, onSuccess);
                    }
            }
.bind(this));
        }
.bind(this), this.onError.bind(this));
    }
    else {
        onSuccess();
    }
}

Depot.prototype.getDefaultValues = function(){
    return {
        personVibrate: true,
        personSound: "system",
        personTonePath: "",
        personToneName: "",
        personBanner: true,
        personBlink: true,
        groupVibrate: true,
        groupSound: "system",
        groupTonePath: "",
        groupToneName: "",
        groupBanner: true,
        groupBlink: true,
        backgroundTimeout: "00:15:00",
        chatTextSize: 16,
        imageResolution: 800,
        phoneTypes: "mobile",
        language: Mojo.Locale.getCurrentLocale(),
        backimageType: "none",
        bubbleclass: "classic",
        closeifnomessages: 0,
		autodownloadImage: false,
		autodownloadVideo: false,
		autodownloadAudio: false,
		autodownloadVoiceNote: false,
		showOnline: true,
		invisibleMode: false
    };
}

Depot.prototype.setDefaultNewValues = function(onSuccess){
    if (!("chatTextSize" in this.prefs)) 
        this.prefs.chatTextSize = 16;
    if (!("imageResolution" in this.prefs)) 
        this.prefs.imageResolution = 800;
    if (!("phoneTypes" in this.prefs)) 
        this.prefs.phoneTypes = "mobile";
    if (!("language" in this.prefs)) 
        this.prefs.language = Mojo.Locale.getCurrentLocale();
    if (!("backimageType" in this.prefs)) 
        this.prefs.backimageType = "none";
    if (!("backimagePath" in this.prefs)) 
        this.prefs.backimagePath = "";
    if (!("bubbleclass" in this.prefs)) 
        this.prefs.bubbleclass = "classic";
    if (!("closeifnomessages" in this.prefs)) 
        this.prefs.closeifnomessages = 0;
    if (this.prefs.backgroundTimeout == "asArrive") 
        this.prefs.backgroundTimeout = "00:05:00";
    if (!("autodownloadImage" in this.prefs)) 
        this.prefs.autodownloadImage = false;
    if (!("autodownloadVideo" in this.prefs)) 
        this.prefs.autodownloadVideo = false;
    if (!("autodownloadAudio" in this.prefs)) 
        this.prefs.autodownloadAudio = false;
    if (!("autodownloadVoiceNote" in this.prefs)) 
        this.prefs.autodownloadVoiceNote = false;
	if (!("showOnline" in this.prefs)) 
        this.prefs.showOnline = true;
	if (!("invisibleMode" in this.prefs))
		this.prefs.invisibleMode = false;
    
    this.putPrefs(this.prefs, undefined, onSuccess);
}



Depot.prototype.getPrefs = function(onSuccess){
    this.db.get("prefs", onSuccess, this.onError.bind(this));
}

Depot.prototype.putPrefs = function(data, value, onSuccess){
    if (value !== undefined) {
        this.prefs[data] = value;
    }
    else {
        this.prefs = data;
    }
    
    if (!onSuccess) 
        onSuccess = function(){
        };
    
    this.db.add("prefs", this.prefs, onSuccess, this.onError.bind(this));
}

Depot.prototype.onError = function(error){
    Mojo.Log.error("Error loading depot: %j", error);
}
