var _mediaDownloads = new HashTable();

function MediaDownloader(){
}

function MediaDownloader(msg){
    this.msg = msg;
    this.serviceRequest = null;
    this.progress = 0;
    this.ticket = null;
    this.downloading = false;
}

MediaDownloader.prototype.requestDownload = function(fileName){
    Mojo.Log.info("download file %j", fileName)
    this.downloading = true;
    this.msg.status = Message.STATUS_MEDIA_DOWNLOADING;
    _appDB.updateMessageStatus(this.msg);
    var useUrl = Media.rewriteDownloadURL(this.msg);
    Mojo.Log.info("Used URL = ", useUrl);
    Mojo.Log.info("mime = ", Media.getMimeType(this.msg, Media.getExt(useUrl)));
    
    this.serviceRequest = new Mojo.Service.Request('palm://com.palm.downloadmanager/', {
        method: "download",
        parameters: {
            target: useUrl,
            mime: Media.getMimeType(this.msg, Media.getExt(useUrl)),
            targetDir: Media.getDir(this.msg),
            targetFilename: (fileName != null && fileName != undefined ? Media.getFileName(fileName) : Media.getDateFormatted() + "-" + Media.selectFilename(this.msg, useUrl)),
            keepFilenameOnRedirect: true,
            subscribe: true
        },
        onSuccess: this.downloadSuccess.bind(this),
        onFailure: this.downloadFailure.bind(this)
    });
}

MediaDownloader.prototype.downloadSuccess = function(response){
    if (response.returnValue) 
        this.ticket = response.ticket;
    
    if (response.amountReceived) {
        this.progress = Math.round((response.amountReceived / this.msg.media_size) * 100);
        Mojo.Log.info("Progress: " + this.progress);
    }
    
    if (response.completed) {
        if (response.completionStatusCode == 200) {
            this.msg.status = Message.STATUS_MEDIA_DOWNLOADED;
            this.msg.downloadedFile = Media.getDir(this.msg) + response.destFile;
            _appDB.updateMediaMessage(this.msg, function(){
                //            if (_openChatAssistant != null)
                //                _openChatAssistant.updateMessageStatus(this.msg);
                _appAssistant.notifyMessageStatusUpdated(this.msg);
                this.removeDownloader();
            }
.bind(this));
        }
        else {
            this.downloadFailure(response);
        }
    }
    else {
        if (_openChatAssistant != null) 
            _openChatAssistant.updateMessageStatus(this.msg);
    }
}

MediaDownloader.prototype.downloadFailure = function(response){
    Mojo.Log.info("Power Manager Service Request:activityEnd FAILED!");
    if (this.msg.downloadedFile == null || this.msg.downloadedFile == "") 
        this.msg.status = Message.STATUS_MEDIA_UPLOADERROR;
    else 
        this.msg.status = Message.STATUS_MEDIA_DOWNLOADED;
    
    _appDB.updateMessageStatus(this.msg, function(){
        //        if (_openChatAssistant != null)
        //            _openChatAssistant.updateMessageStatus(this.msg);
        _appAssistant.notifyMessageStatusUpdated(this.msg);
        this.removeDownloader();
    }
.bind(this));
    
}

MediaDownloader.prototype.stopDownload = function(){
    this.serviceRequest = new Mojo.Service.Request('palm://com.palm.downloadmanager/', {
        method: "cancelDownload",
        parameters: {
            ticket: this.ticket
        },
        onSuccess: function(r){
            Mojo.Log.info("Download canceled success: %j", r);
            if (this.msg.downloadedFile == null || this.msg.downloadedFile == "") 
                this.msg.status = Message.STATUS_NONE;
            else 
                this.msg.status = Message.STATUS_MEDIA_DOWNLOADED;
            _appDB.updateMessageStatus(this.msg, function(){
                //                if (_openChatAssistant != null)
                //                    _openChatAssistant.updateMessageStatus(this.msg);
                _appAssistant.notifyMessageStatusUpdated(this.msg);
                this.removeDownloader();
            }
.bind(this));
        }
.bind(this)        ,
        onFailure: function(r){
            Mojo.Log.info("Download canceled failure: %j", r);
        }
    });
}

MediaDownloader.prototype.removeDownloader = function(){
    this.msg.mediaDownloader = null;
    this.downloading = false;
    _mediaDownloads.removeItem(this.msg.keyString());
    delete this;
}

MediaDownloader.download = function(msg){
    if (_mediaDownloads.getItem(msg.keyString()) !== undefined) {
        msg.mediaDownloader = _mediaDownloads.getItem(msg.keyString());
        msg.mediaDownloader.msg = msg;
    }
    else {
        msg.mediaDownloader = new MediaDownloader(msg);
        _mediaDownloads.setItem(msg.keyString(), msg.mediaDownloader);
    }
    if (!msg.mediaDownloader.downloading) 
        msg.mediaDownloader.requestDownload(msg.downloadedFile);
}

MediaDownloader.stopDownload = function(msg){
    if (_mediaDownloads.getItem(msg.keyString()) !== undefined) {
        msg.mediaDownloader = _mediaDownloads.getItem(msg.keyString());
        msg.mediaDownloader.msg = msg;
        if (msg.mediaDownloader.downloading) 
            msg.mediaDownloader.stopDownload();
    }
}

MediaDownloader.existsDownloader = function(msg){
    if (_mediaDownloads.getItem(msg.keyString()) !== undefined) {
        return _mediaDownloads.getItem(msg.keyString());
    }
    return null;
}

MediaDownloader.autoDownload = function(msg){
    if (!msg.from_me) {
        var autodownload = false;
        switch (msg.media_wa_type) {
            case Message.WA_TYPE_IMAGE:
                autodownload = _depotDB.prefs.autodownloadImage;
                break;
            case Message.WA_TYPE_VIDEO:
                autodownload = _depotDB.prefs.autodownloadVideo;
                break;
            case Message.WA_TYPE_AUDIO:
                autodownload = _depotDB.prefs.autodownloadAudio;
                break;
            case Message.WA_TYPE_AUDIO_LIVE:
                autodownload = _depotDB.prefs.autodownloadVoiceNote;
                break;
        }
        
        if (autodownload && (msg.status != Message.STATUS_MEDIA_DOWNLOADED)) {
            MediaDownloader.download(msg);
        }
    }
}
