var _mediaUploads = new HashTable();

function MediaUploader(){
}

function MediaUploader(msg, file, recipients){
    this.msg = msg;
    this.serviceRequest = null;
    this.progress = 0;
    this.ticket = null;
    this.uploading = false;
    this.file = file;
    this.recipients = recipients;
}


MediaUploader.notifyUpdateStatus = function(msg){
    _appAssistant.notifyMessageStatusUpdated(msg);
    if (msg.remote_jid.indexOf("broadcast") == 0) {
        _appAssistant.notifyUpdateBroadcast(msg);
    }
}

MediaUploader.prototype.requestUpload = function(){
    Mojo.Log.info("upload request %j", this.file);
    this.uploading = true;
    this.msg.status = Message.STATUS_MEDIA_UPLOADING;
    _appDB.updateMessageStatus(this.msg, function(){
        MediaUploader.notifyUpdateStatus(this.msg);
    }
.bind(this));
    
    
    _mojowhatsupPlugin.safePluginCall(function(){
        _plugin.sendMediaUploadRequest(this.msg.keyString(), this.file, Message.getTypeString(this.msg.media_wa_type), _depotDB.prefs.imageResolution); // Media.getMimeType(this.msg, Media.getExt(file)));		
    }
.bind(this));
}

MediaUploader.prototype.uploadFile = function(url){
    _mojowhatsupPlugin.safePluginCall(function(){
        Mojo.Log.info("uploadFile file %s url %s", this.file, url);
        _plugin.uploadFile(this.msg.keyString(), url, Media.getMimeType(this.msg, Media.getExt(this.file)));
    }
.bind(this));
}

MediaUploader.prototype.uploadSuccess = function(response){
    Mojo.Log.info("Uploader Media success: %j", response);
    
    if (response.completed == true) {
        var jsonResponse = MojowhatsupPluginModel.jsonParse(response.response);
        this.msg.media_url = jsonResponse.url;
        this.msg.media_name = jsonResponse.name;
        this.msg.media_size = jsonResponse.size;
        
        if (this.msg.media_url) {
            this.msg.status = Message.STATUS_UNSENT;
            this.msg.timestamp = new Date().getTime();
            if (this.recipients) {
                var timeout = 1;
                for (var i = 0; i < this.recipients.length; i++) {
                    var jid = this.recipients[i];
                    var msgSend = this.msg.cloneToSend(jid);
                    msgSend.status = Message.STATUS_UNSENT;
                    msgSend.downloadedFile = this.file;
                    setTimeout(function(msgSend){
                        _mojowhatsupPlugin.messageForMe(msgSend, true, function(msgSend){
                            _mojowhatsupPlugin.safePluginCall(function(){
                                _plugin.sendMessage(msgSend.toJSONString());
                            });
                        });
                    }, timeout, msgSend);
                    timeout += 100;
                }
                _appAssistant.notifyUpdateChats(undefined);
                this.msg.mediaUploader = null;
                this.removeUploader();
            }
            else {
                _appDB.updateMediaMessage(this.msg, function(){
                    _mojowhatsupPlugin.safePluginCall(function(){
                        this.msg.mediaUploader = null;
                        MediaUploader.notifyUpdateStatus(this.msg);
                        if (this.msg.remote_jid.indexOf("broadcast") == 0) {
                            _plugin.sendBroadcastMessage(this.msg.remote_resource, this.msg.toJSONString());
                        }
                        else {
                            _appAssistant.notifyUpdateChats(undefined);
                            _plugin.sendMessage(this.msg.toJSONString());
                        }
                        this.removeUploader();
                    }
.bind(this));
                }
.bind(this));
            }
        }
        else {
            this.uploadFailure(response);
        }
    }
    else 
        if (response.completed == false) {
            this.progress = response.progress;
            if (this.msg.media_size == null && response.size) 
                this.msg.media_size = response.size;
            MediaUploader.notifyUpdateStatus(this.msg);
        }
}

MediaUploader.prototype.uploadFailure = function(response){
    Mojo.Log.error("Uploader Media failure!, %j", response);
    this.msg.status = Message.STATUS_MEDIA_UPLOADERROR;
    
    if (this.recipients) {
        for (var i = 0; i < this.recipients.length; i++) {
            var jid = this.recipients[i];
            var msgSend = this.msg.cloneToSend(jid);
            msgSend.status = Message.STATUS_MEDIA_UPLOADERROR;
            msgSend.downloadedFile = this.file;
            _mojowhatsupPlugin.messageForMe(msgSend, true);
        }
        _appAssistant.notifyUpdateChats(undefined);
        this.msg.mediaUploader = null;
        this.removeUploader();
    }
    else {
        _appDB.updateMessageStatus(this.msg, function(){
            MediaUploader.notifyUpdateStatus(this.msg);
            this.removeUploader();
        }
.bind(this));
    }
}

MediaUploader.prototype.stopUpload = function(){
    _mojowhatsupPlugin.safePluginCall(function(){
        _plugin.stopUploadRequest(this.msg.keyString());
        this.msg.status = Message.STATUS_NONE;
        _appDB.updateMessageStatus(this.msg, function(){
            MediaUploader.notifyUpdateStatus(this.msg);
            this.removeUploader();
        }
.bind(this));
    }
.bind(this));
}

MediaUploader.prototype.removeUploader = function(){
    this.msg.mediaUploader = null;
    this.uploading = false;
    _mediaUploads.removeItem(this.msg.keyString());
    delete this;
}

MediaUploader.upload = function(msg, file, recipients){
    if (_mediaUploads.getItem(msg.keyString()) !== undefined) {
        msg.mediaUploader = _mediaUploads.getItem(msg.keyString());
        msg.mediaUploader.msg = msg;
    }
    else {
        msg.mediaUploader = new MediaUploader(msg, file, recipients);
        _mediaUploads.setItem(msg.keyString(), msg.mediaUploader);
    }
    if (!msg.mediaUploader.uploading) 
        msg.mediaUploader.requestUpload();
}

MediaUploader.stopUpload = function(msg){
    if (_mediaUploads.getItem(msg.keyString()) !== undefined) {
        msg.mediaUploader = _mediaUploads.getItem(msg.keyString());
        msg.mediaUploader.msg = msg;
        if (msg.mediaUploader.uploading) 
            msg.mediaUploader.stopUpload();
    }
    else {
        msg.status = Message.STATUS_NONE;
        _appDB.updateMessageStatus(msg, function(msg){
            MediaUploader.notifyUpdateStatus(msg);
        }
.bind(this));
    }
}

MediaUploader.existsUploader = function(msg){
    if (_mediaUploads.getItem(msg.keyString()) !== undefined) {
        return _mediaUploads.getItem(msg.keyString());
    }
    return null;
}

MediaUploader.getUploader = function(msgId){
    return _mediaUploads.getItem(msgId);
}
